@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2025 Firejail Authors
+ * Copyright (C) 2014-2026 Firejail Authors
  *
  * This file is part of firejail project
  *
@@ -73,9 +73,6 @@ int arg_debug_whitelists = 0;			// print debug messages for whitelists
 int arg_debug_private_lib = 0;			// print debug messages for private-lib
 int arg_nonetwork = 0;				// --net=none
 int arg_command = 0;				// -c
-int arg_overlay = 0;				// overlay option
-int arg_overlay_keep = 0;			// place overlay diff in a known directory
-int arg_overlay_reuse = 0;			// allow the reuse of overlays
 
 int arg_landlock_enforce = 0;		// enforce the Landlock ruleset
 
@@ -94,12 +91,12 @@ char *arg_caps_list = NULL;			// optional caps list
 int arg_trace = 0;				// syscall tracing support
 char *arg_tracefile = NULL;			// syscall tracing file
 int arg_tracelog = 0;				// blacklist tracing support
+int arg_rlimit_as = 0;				// rlimit as
 int arg_rlimit_cpu = 0;				// rlimit max cpu time
+int arg_rlimit_fsize = 0;				// rlimit fsize
 int arg_rlimit_nofile = 0;			// rlimit nofile
 int arg_rlimit_nproc = 0;			// rlimit nproc
-int arg_rlimit_fsize = 0;				// rlimit fsize
 int arg_rlimit_sigpending = 0;			// rlimit fsize
-int arg_rlimit_as = 0;				// rlimit as
 int arg_nogroups = 0;				// disable supplementary groups
 #ifdef HAVE_FORCE_NONEWPRIVS
 int arg_nonewprivs = 1;			// set the NO_NEW_PRIVS prctl
@@ -172,6 +169,7 @@ int login_shell = 0;
 int just_run_the_shell = 0;
 int arg_netlock = 0;
 int arg_restrict_namespaces = 0;
+int arg_allow_bwrap = 0;
 
 int parent_to_child_fds[2];
 int child_to_parent_fds[2];
@@ -375,23 +373,6 @@ static void run_cmd_and_exit(int i, int argc, char **argv) {
 		usage();
 		exit(0);
 	}
-	else if (strcmp(argv[i], "--version") == 0) {
-		print_version_full();
-		exit(0);
-	}
-#ifdef HAVE_OVERLAYFS
-	else if (strcmp(argv[i], "--overlay-clean") == 0) {
-		if (checkcfg(CFG_OVERLAYFS)) {
-			if (remove_overlay_directory()) {
-				fprintf(stderr, "Error: cannot remove overlay directory\n");
-				exit(1);
-			}
-		}
-		else
-			exit_err_feature("overlayfs");
-		exit(0);
-	}
-#endif
 #ifdef HAVE_X11
 	else if (strcmp(argv[i], "--x11") == 0) {
 		if (checkcfg(CFG_X11)) {
@@ -1075,41 +1056,12 @@ int main(int argc, char **argv, char **envp) {
 	// check standard streams before opening any file
 	fix_std_streams();
 
-	// initialize values from firejail.config (needed for arg_max_count)
-	checkcfg(0);
-
 	// argument count should be larger than 0
 	if (argc == 0 || !argv || strlen(argv[0]) == 0) {
 		fprintf(stderr, "Error: argv is invalid\n");
 		exit(1);
-	} else if (argc >= arg_max_count) {
-		fprintf(stderr, "Error: too many arguments: argc (%d) >= arg-max-count (%d)\n",
-		        argc, arg_max_count);
-		exit(1);
 	}
 
-	// sanity check for arguments
-	for (i = 0; i < argc; i++) {
-		if (strlen(argv[i]) >= arg_max_len) {
-			fprintf(stderr, "Error: too long argument: argv[%d] len (%zu) >= arg-max-len (%lu): '%s'\n",
-			        i, strlen(argv[i]), arg_max_len, argv[i]);
-			exit(1);
-		}
-	}
-
-	// Stash environment variables
-	for (i = 0, ptr = envp; ptr && *ptr && i < MAX_ENVS; i++, ptr++)
-		env_store(*ptr, SETENV);
-
-	// sanity check for environment variables
-	if (i >= MAX_ENVS) {
-		fprintf(stderr, "Error: too many environment variables: >= MAX_ENVS (%d)\n", MAX_ENVS);
-		exit(1);
-	}
-
-	// Reapply a minimal set of environment variables
-	env_apply_whitelist();
-
 	// process --quiet
 	const char *env_quiet = env_get("FIREJAIL_QUIET");
 	if (check_arg(argc, argv, "--quiet", 1) || (env_quiet && strcmp(env_quiet, "yes") == 0))
@@ -1121,6 +1073,44 @@ int main(int argc, char **argv, char **envp) {
 		arg_quiet = 0;
 	}
 
+	// process --version
+	if (check_arg(argc, argv, "--version", 1)) {
+		print_version_full();
+		exit(0);
+	}
+
+	// initialize values from firejail.config (needed for arg/env checks)
+	checkcfg(0);
+
+	// sanity check for arguments
+	if (argc >= arg_max_count) {
+		fprintf(stderr, "Error: too many arguments: argc (%d) >= arg-max-count (%d)\n",
+		        argc, arg_max_count);
+		exit(1);
+	}
+
+	for (i = 0; i < argc; i++) {
+		if (strlen(argv[i]) >= arg_max_len) {
+			fprintf(stderr, "Error: too long argument: argv[%d] len (%zu) >= arg-max-len (%lu): '%s'\n",
+			        i, strlen(argv[i]), arg_max_len, argv[i]);
+			exit(1);
+		}
+	}
+
+	// Stash environment variables
+	for (i = 0, ptr = envp; ptr && *ptr && i < env_max_count; i++, ptr++)
+		env_store(*ptr, SETENV);
+
+	// sanity check for environment variables
+	if (i >= env_max_count) {
+		fprintf(stderr, "Error: too many environment variables: >= env-max-count (%d)\n",
+		        env_max_count);
+		exit(1);
+	}
+
+	// Reapply a minimal set of environment variables
+	env_apply_whitelist();
+
 	// check if the user is allowed to use firejail
 	init_cfg(argc, argv);
 
@@ -1141,11 +1131,6 @@ int main(int argc, char **argv, char **envp) {
 		int rv = check_kernel_procs();
 		EUID_USER();
 		if (rv == 0) {
-			if (check_arg(argc, argv, "--version", 1)) {
-				print_version_full();
-				exit(0);
-			}
-
 			// start the program directly without sandboxing
 			run_no_sandbox(argc, argv);
 			__builtin_unreachable();
@@ -1159,19 +1144,6 @@ int main(int argc, char **argv, char **envp) {
 	if (check_arg(argc, argv, "--build", 0)) // supports both --build and --build=filename
 		run_builder(argc, argv); // this function will not return
 
-	// intrusion detection system
-#ifdef HAVE_IDS
-	if (check_arg(argc, argv, "--ids-", 0)) // supports both --ids-init and --ids-check
-		run_ids(argc, argv); // this function will not return
-#else
-	if (check_arg(argc, argv, "--ids-", 0)) { // supports both --ids-init and --ids-check
-		fprintf(stderr, "Error: IDS features disabled in your Firejail build.\n"
-			"\tTo enable it, configure your build system using --enable-ids.\n"
-			"\tExample: ./configure --prefix=/usr --enable-ids\n\n");
-		exit(1);
-	}
-#endif
-
 	EUID_ROOT();
 #ifndef HAVE_SUID
 	if (geteuid() != 0) {
@@ -1357,6 +1329,8 @@ int main(int argc, char **argv, char **envp) {
 		if (!cmd)
 			errExit("strdup");
 		profile_add(cmd);
+		// disable apparmor - our apparmor profile disables ptrace by default!
+		profile_add_ignore("apparmor");
 	}
 
 	// for appimages we need to remove "include disable-shell.inc from the profile
@@ -1396,6 +1370,8 @@ int main(int argc, char **argv, char **envp) {
 		else if (strcmp(argv[i], "--allow-debuggers") == 0) {
 			// already handled
 		}
+		else if (strcmp(argv[i], "--allow-bwrap") == 0)
+			arg_allow_bwrap = 1;
 
 
 		//*************************************
@@ -1409,6 +1385,12 @@ int main(int argc, char **argv, char **envp) {
 			else
 				exit_err_feature("x11");
 		}
+		else if (strncmp(argv[i], "--xephyr-extra-params=", 16) == 0) {
+			if (checkcfg(CFG_X11))
+				; // the processing is done directly in x11.c
+			else
+				exit_err_feature("x11");
+		}
 #endif
 		//*************************************
 		// filtering
@@ -1634,42 +1616,44 @@ int main(int argc, char **argv, char **envp) {
 			else
 				exit_err_feature("tracelog");
 		}
+		else if (strncmp(argv[i], "--rlimit-as=", 12) == 0) {
+			cfg.rlimit_as = parse_arg_size(argv[i] + 12);
+			if (cfg.rlimit_as == 0) {
+				fprintf(stderr, "Error: invalid rlimit-as: %s; use only positive numbers and K, M or G suffix\n",
+				        argv[i] + 12);
+				exit(1);
+			}
+			arg_rlimit_as = 1;
+		}
 		else if (strncmp(argv[i], "--rlimit-cpu=", 13) == 0) {
-			check_unsigned(argv[i] + 13, "Error: invalid rlimit");
+			check_unsigned(argv[i] + 13, "Error: invalid rlimit-cpu");
 			sscanf(argv[i] + 13, "%llu", &cfg.rlimit_cpu);
 			arg_rlimit_cpu = 1;
 		}
-		else if (strncmp(argv[i], "--rlimit-nofile=", 16) == 0) {
-			check_unsigned(argv[i] + 16, "Error: invalid rlimit");
-			sscanf(argv[i] + 16, "%llu", &cfg.rlimit_nofile);
-			arg_rlimit_nofile = 1;
-		}
-		else if (strncmp(argv[i], "--rlimit-nproc=", 15) == 0) {
-			check_unsigned(argv[i] + 15, "Error: invalid rlimit");
-			sscanf(argv[i] + 15, "%llu", &cfg.rlimit_nproc);
-			arg_rlimit_nproc = 1;
-		}
 		else if (strncmp(argv[i], "--rlimit-fsize=", 15) == 0) {
 			cfg.rlimit_fsize = parse_arg_size(argv[i] + 15);
 			if (cfg.rlimit_fsize == 0) {
-				perror("Error: invalid rlimit-fsize. Only use positive numbers and k, m or g suffix.");
+				fprintf(stderr, "Error: invalid rlimit-fsize: %s; use only positive numbers and K, M or G suffix\n",
+				        argv[i] + 15);
 				exit(1);
 			}
 			arg_rlimit_fsize = 1;
 		}
+		else if (strncmp(argv[i], "--rlimit-nofile=", 16) == 0) {
+			check_unsigned(argv[i] + 16, "Error: invalid rlimit-nofile");
+			sscanf(argv[i] + 16, "%llu", &cfg.rlimit_nofile);
+			arg_rlimit_nofile = 1;
+		}
+		else if (strncmp(argv[i], "--rlimit-nproc=", 15) == 0) {
+			check_unsigned(argv[i] + 15, "Error: invalid rlimit-nproc");
+			sscanf(argv[i] + 15, "%llu", &cfg.rlimit_nproc);
+			arg_rlimit_nproc = 1;
+		}
 		else if (strncmp(argv[i], "--rlimit-sigpending=", 20) == 0) {
-			check_unsigned(argv[i] + 20, "Error: invalid rlimit");
+			check_unsigned(argv[i] + 20, "Error: invalid rlimit-sigpending");
 			sscanf(argv[i] + 20, "%llu", &cfg.rlimit_sigpending);
 			arg_rlimit_sigpending = 1;
 		}
-		else if (strncmp(argv[i], "--rlimit-as=", 12) == 0) {
-			cfg.rlimit_as = parse_arg_size(argv[i] + 12);
-			if (cfg.rlimit_as == 0) {
-				perror("Error: invalid rlimit-as. Only use positive numbers and k, m or g suffix.");
-				exit(1);
-			}
-			arg_rlimit_as = 1;
-		}
 		else if (strncmp(argv[i], "--ipc-namespace", 15) == 0)
 			arg_ipc = 1;
 		else if (strncmp(argv[i], "--cpu=", 6) == 0)
@@ -1786,78 +1770,6 @@ int main(int argc, char **argv, char **envp) {
 		}
 		else if (strcmp(argv[i], "--disable-mnt") == 0)
 			arg_disable_mnt = 1;
-#ifdef HAVE_OVERLAYFS
-		else if (strcmp(argv[i], "--overlay") == 0) {
-			if (checkcfg(CFG_OVERLAYFS)) {
-				if (arg_overlay) {
-					fprintf(stderr, "Error: only one overlay command is allowed\n");
-					exit(1);
-				}
-
-				if (cfg.chrootdir) {
-					fprintf(stderr, "Error: --overlay and --chroot options are mutually exclusive\n");
-					exit(1);
-				}
-				arg_overlay = 1;
-				arg_overlay_keep = 1;
-
-				char *subdirname;
-				if (asprintf(&subdirname, "%d", getpid()) == -1)
-					errExit("asprintf");
-				cfg.overlay_dir = fs_check_overlay_dir(subdirname, arg_overlay_reuse);
-
-				free(subdirname);
-			}
-			else
-				exit_err_feature("overlayfs");
-		}
-		else if (strncmp(argv[i], "--overlay-named=", 16) == 0) {
-			if (checkcfg(CFG_OVERLAYFS)) {
-				if (arg_overlay) {
-					fprintf(stderr, "Error: only one overlay command is allowed\n");
-					exit(1);
-				}
-				if (cfg.chrootdir) {
-					fprintf(stderr, "Error: --overlay and --chroot options are mutually exclusive\n");
-					exit(1);
-				}
-				arg_overlay = 1;
-				arg_overlay_keep = 1;
-				arg_overlay_reuse = 1;
-
-				char *subdirname = argv[i] + 16;
-				if (*subdirname == '\0') {
-					fprintf(stderr, "Error: invalid overlay option\n");
-					exit(1);
-				}
-
-				// check name
-				invalid_filename(subdirname, 0); // no globbing
-				if (strstr(subdirname, "..") || strstr(subdirname, "/")) {
-					fprintf(stderr, "Error: invalid overlay name\n");
-					exit(1);
-				}
-				cfg.overlay_dir = fs_check_overlay_dir(subdirname, arg_overlay_reuse);
-			}
-			else
-				exit_err_feature("overlayfs");
-		}
-		else if (strcmp(argv[i], "--overlay-tmpfs") == 0) {
-			if (checkcfg(CFG_OVERLAYFS)) {
-				if (arg_overlay) {
-					fprintf(stderr, "Error: only one overlay command is allowed\n");
-					exit(1);
-				}
-				if (cfg.chrootdir) {
-					fprintf(stderr, "Error: --overlay and --chroot options are mutually exclusive\n");
-					exit(1);
-				}
-				arg_overlay = 1;
-			}
-			else
-				exit_err_feature("overlayfs");
-		}
-#endif
 		else if (strncmp(argv[i], "--include=", 10) == 0) {
 			char *ppath = expand_macros(argv[i] + 10);
 			if (!ppath)
@@ -1958,11 +1870,6 @@ int main(int argc, char **argv, char **envp) {
 #ifdef HAVE_CHROOT
 		else if (strncmp(argv[i], "--chroot=", 9) == 0) {
 			if (checkcfg(CFG_CHROOT)) {
-				if (arg_overlay) {
-					fprintf(stderr, "Error: --overlay and --chroot options are mutually exclusive\n");
-					exit(1);
-				}
-
 				// extract chroot dirname
 				cfg.chrootdir = expand_macros(argv[i] + 9);
 				if (*cfg.chrootdir == '\0') {
@@ -2876,13 +2783,11 @@ int main(int argc, char **argv, char **envp) {
 	}
 	EUID_ASSERT();
 
-	// exit chroot, overlay and appimage sandboxes when caps are explicitly specified on command line
+	// exit chroot and appimage sandboxes when caps are explicitly specified on command line
 	if (getuid() != 0 && arg_caps_cmdline) {
 		char *opt = NULL;
 		if (arg_appimage)
 			opt = "appimage";
-		else if (arg_overlay)
-			opt = "overlay";
 		else if (cfg.chrootdir)
 			opt = "chroot";
 
@@ -2901,11 +2806,7 @@ int main(int argc, char **argv, char **envp) {
 
 	// check user namespace (--noroot) options
 	if (arg_noroot) {
-		if (arg_overlay) {
-			fwarning("--overlay and --noroot are mutually exclusive, --noroot disabled...\n");
-			arg_noroot = 0;
-		}
-		else if (cfg.chrootdir) {
+		if (cfg.chrootdir) {
 			fwarning("--chroot and --noroot are mutually exclusive, --noroot disabled...\n");
 			arg_noroot = 0;
 		}
@@ -3045,11 +2946,7 @@ int main(int argc, char **argv, char **envp) {
 	}
 	EUID_ASSERT();
 
-	if (arg_noroot && arg_overlay) {
-		fwarning("--overlay and --noroot are mutually exclusive, noroot disabled\n");
-		arg_noroot = 0;
-	}
-	else if (arg_noroot && cfg.chrootdir) {
+	if (arg_noroot && cfg.chrootdir) {
 		fwarning("--chroot and --noroot are mutually exclusive, noroot disabled\n");
 		arg_noroot = 0;
 	}
